using System;
using System.Collections.Generic;
using System.Text;

namespace TecnoComm {

    public class deviceRS232 : device {

        private System.IO.Ports.SerialPort port;
        private string[] portNames=new string[0];
        

        // List devices of this type that are still available
        // RETURN: list of devices
        public string[] listDevices() {
            portNames = System.IO.Ports.SerialPort.GetPortNames();
            for (int i = 0; i < portNames.Length - 1; i++) {
                for (int j = i + 1; j < portNames.Length; j++) {
                    if (portNames[i].CompareTo(portNames[j])>0) {
                        string sw = portNames[i];
                        portNames[i] = portNames[j];
                        portNames[j] = sw;
                    }
                }
            }
            return portNames;
        }

        uint baudRate;

        // Open a device for communication
        // device: identify device to open: should be the index (0 based) of the device
        //         in the list obtained by listDevices
        // baud  : baud rate required for communication; other serial parameters are
        //         fixed: 8N1.
        // RETURN: true on success
        public bool open(int device, uint baud) {
            close();
            try {
                port = new System.IO.Ports.SerialPort(portNames[device], (int)baud);
                port.Open();
                port.ReadTimeout=0;
                port.DtrEnable = true;
                baudRate = baud;
                return true;
            }
            catch (System.Exception) {
            }
            return false;
        }

        // Close device and releases it
        // RETURN: true on success
        public bool close() {
            if (port==null) return true;
            if (port.IsOpen) {
                port.DtrEnable=false;
                port.Close();
            }
            port = null;
            return true;
        }

        private const uint outBufferSize = 64; //use 1 for debugging (64)
        private byte[] outBuffer = new byte[outBufferSize];
        private uint outBufferNext;

        private bool sendOutBuffer() {
            if (outBufferNext == 0) return true;

            //port.Write(outBuffer, 0, (int)outBufferNext);
            /// inBufferLength = 0;

            for (int i = 0; i < outBufferNext; i++) {
                port.Write(outBuffer, i, 1);

                /*
                uint sz=0;
                while (sz == 0) {
                    try {
                        sz = (uint)port.Read(inBuffer, (int)inBufferLength, 1);
                        inBufferLength += sz;
                    }
                    catch (System.TimeoutException) { }
                }
                 */
                

            }
            outBufferNext = 0;
            return true;
        }

        // Sends a byte on the communication channel
        // b     : byte to send
        // RETURN: true on success
        public bool putByte(byte b) {
            if (port==null) return false;
            outBuffer[outBufferNext++] = b;
            if (outBufferNext == outBufferSize) return sendOutBuffer();
            else return true;
        }

        private const uint inBufferSize = 64;
        private byte[] inBuffer = new byte[inBufferSize];
        private uint inBufferNext;
        private uint inBufferLength;

        private void fillInBuffer() {
            inBufferNext = 0;
            try {
                inBufferLength = (uint)port.Read(inBuffer, 0, (int)inBufferSize);
            }
            catch (System.TimeoutException) {
                inBufferLength = 0;
            }
        }

        // Get a byte from the communication channel
        // b     : if a byte is available, it is stored here
        // RETURN: true if a byte was retrieved
        public bool getByte(ref byte b) {
            if (port == null) return false;
            if (!sendOutBuffer()) return false;
            if (inBufferNext >= inBufferLength) {
                fillInBuffer();
                if (inBufferNext >= inBufferLength) return false;
            }
            b = inBuffer[inBufferNext++];
            return true;
        }

        // Empty input and output buffers
        // RETURN: true on success
        public bool flush() {
            if (port == null) return false;
            port.DiscardOutBuffer();
            port.DiscardInBuffer();
            inBufferNext = inBufferLength = 0;
            outBufferNext = 0;
            return true;
        }

        // Synchronizes communication speed
        // RETURN: true synchronization succeeded
        public bool synchronize() {            
            /*
            if (port == null) return false;
            flush();
            port.BaudRate = (int)(baudRate / 4);
            byte[] buffer=new byte[1];
            port.Write(buffer, 0, 1);
            int start = System.Environment.TickCount;
            while (System.Environment.TickCount - start <= 1000) ;
            port.BaudRate = (int)baudRate;
             * */
            return true;
        }

        // Signal if the PC is ready (should map to DTR)
        // on: set to true to signal ready
        public bool signal(bool on) {
            if (port == null) return false;
            port.DtrEnable = on;
            return true;
        }

    }
}
