using System;
using System.Collections.Generic;
using System.Text;

using System.Security.Permissions;
using System.Runtime.InteropServices;

namespace TecnoComm {

    public class deviceUSB : device {
        [DllImport("FTD2XX.dll", EntryPoint = "FT_ListDevices", ExactSpelling = false, SetLastError = true)]
        private static extern unsafe uint FT_ListDevices(void* pArg1, void* pArg2, uint Flags);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_Open", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_Open(int deviceNumber, uint* pHandle);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_ResetDevice", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_ResetDevice(uint ftHandle);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_SetBaudRate", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_SetBaudRate(uint ftHandle, uint BaudRate);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_SetDataCharacteristics", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_SetDataCharacteristics(uint ftHandle, byte WordLength, byte StopBits, byte Parity);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_SetFlowControl", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_SetFlowControl(uint ftHandle, ushort FlowControl, byte XonChar, byte XoffChar);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_SetTimeouts", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_SetTimeouts(uint ftHandle, uint ReadTimeout, uint WriteTimeout);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_Close", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_Close(uint ftHandle);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_Write", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_Write(uint ftHandle, byte* lpBuffer, uint nBufferSize, uint* lpBytesWritten);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_Read", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_Read(uint ftHandle, byte* lpBuffer, uint nBufferSize, uint* lpBytesReturned);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_Purge", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_Purge(uint ftHandle, uint Mask);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_ClrDtr", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_ClrDtr(uint ftHandle);

        [DllImport("FTD2XX.dll", EntryPoint = "FT_SetDtr", ExactSpelling = false, SetLastError = true)]
        static extern unsafe uint FT_SetDtr(uint ftHandle);

        // Handle to FTDI USB device        

        private const uint FT_LIST_NUMBER_ONLY = 0x80000000;
        private const uint FT_LIST_BY_INDEX = 0x40000000;
        private const uint FT_LIST_ALL = 0x20000000;

        private const uint FT_OPEN_BY_SERIAL_NUMBER = 1;
        private const uint FT_OPEN_BY_DESCRIPTION = 2;
        private const uint FT_OPEN_BY_LOCATION = 4;

        private const uint FT_OK = 0;

        private const byte FT_BITS_8 = 8;
        private const byte FT_BITS_7 = 7;
        private const byte FT_BITS_6 = 6;
        private const byte FT_BITS_5 = 5;

        private const byte FT_STOP_BITS_1 = 0;
        private const byte FT_STOP_BITS_1_5 = 1;
        private const byte FT_STOP_BITS_2 = 2;

        private const byte FT_PARITY_NONE = 0;
        private const byte FT_PARITY_ODD = 1;
        private const byte FT_PARITY_EVEN = 2;
        private const byte FT_PARITY_MARK = 3;
        private const byte FT_PARITY_SPACE = 4;

        private const ushort FT_FLOW_NONE = 0x0000;
        private const ushort FT_FLOW_RTS_CTS = 0x0100;
        private const ushort FT_FLOW_DTR_DSR = 0x0200;
        private const ushort FT_FLOW_XON_XOFF = 0x0400;

        private const uint FT_PURGE_RX = 1;
        private const uint FT_PURGE_TX = 2;


        private uint ftHandle;
        uint baud;

        ~deviceUSB() {
            close();
        }

        private unsafe string[] UlistDevices() {
            uint n;
            byte[] s = new byte[64];
            List<string> sl = new List<string>();

            if (FT_ListDevices(&n, null, FT_LIST_NUMBER_ONLY) == FT_OK && n != 0) {
                for (uint i = 0; i < n + 1; i++) {
                    fixed (byte* ps = s) {
                        if (FT_ListDevices((void*)i, ps, FT_LIST_BY_INDEX | FT_OPEN_BY_DESCRIPTION) == FT_OK) {
                            StringBuilder sb = new StringBuilder();
                            for (int j = 0; j < s.Length && s[j] != 0; j++) sb.Append((Char)(s[j]));
                            sl.Add(sb.ToString());
                        }
                    }
                }
            }
            return sl.ToArray();
        }

        // List devices of this type that are still available
        // RETURN: list of devices
        public string[] listDevices() {
            return UlistDevices();
        }

        private unsafe bool Uopen(int device, uint baud) {
            close();
            ftHandle = 0;

            uint newHandle = 0;
            if (FT_Open(device, &newHandle) != FT_OK) return false;
            if (FT_ResetDevice(newHandle) != FT_OK) return false;
            if (FT_SetBaudRate(newHandle, baud) != FT_OK) return false;
            if (FT_SetDataCharacteristics(newHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE) != FT_OK) return false;
            if (FT_SetFlowControl(newHandle, FT_FLOW_NONE, 0, 0) != FT_OK) return false;
            if (FT_SetTimeouts(newHandle, 50, 50) != FT_OK) return false;
            if (FT_SetDtr(newHandle) != FT_OK) return false;

            this.baud = baud;
            ftHandle = newHandle;
            return true;
        }

        // Open a device for communication
        // device: identify device to open: should be the index (0 based) of the device
        //         in the list obtained by listDevices
        // baud  : baud rate required for communication; other serial parameters are
        //         fixed: 8N1.
        // RETURN: true on success
        public bool open(int device, uint baud) {
            return Uopen(device, baud);
        }

        // Close device and releases it
        // RETURN: true on success
        public bool close() {
            if (ftHandle != 0) {
                if (FT_SetDtr(ftHandle) != FT_OK) return false;
                if (FT_Close(ftHandle) != FT_OK) return false;
                ftHandle = 0;
            }
            return true;
        }

        private const uint outBufferSize = 64;
        private byte[] outBuffer = new byte[outBufferSize];
        private uint outBufferNext;

        private unsafe bool sendOutBuffer() {
            uint written;

            if (outBufferNext == 0) return true;

            bool done;
            fixed (byte* pb = outBuffer) {
                done = FT_Write(ftHandle, pb, outBufferNext, &written) == FT_OK && written == outBufferNext;
                outBufferNext = 0;
            }
            return done;
        }

        // Sends a byte on the communication channel
        // b     : byte to send
        // RETURN: true on success
        public bool putByte(byte b) {
            if (ftHandle == 0) return false;
            outBuffer[outBufferNext++] = b;
            if (outBufferNext == outBufferSize) return sendOutBuffer();
            else return true;
        }

        private const uint inBufferSize = 64;
        private byte[] inBuffer = new byte[inBufferSize];
        private uint inBufferNext;
        private uint inBufferLength;

        private unsafe void fillInBuffer() {
            uint read;

            inBufferNext = 0;
            fixed (byte* pb = inBuffer) {
                if (FT_Read(ftHandle, pb, inBufferSize, &read) == FT_OK) {
                    inBufferLength = read;
                }
                else inBufferLength = 0;
                inBufferNext = 0;
            }
        }

        // Get a byte from the communication channel
        // b     : if a byte is available, it is stored here
        // RETURN: true if a byte was retrieved
        public bool getByte(ref byte b) {
            if (ftHandle == 0) return false;
            if (!sendOutBuffer()) return false;
            if (inBufferNext >= inBufferLength) {
                fillInBuffer();
                if (inBufferNext >= inBufferLength) return false;
            }
            b = inBuffer[inBufferNext++];
            return true;
        }

        // Empty input and output buffers
        // RETURN: true on success
        public bool flush() {
            if (ftHandle == 0) return false;
            return FT_Purge(ftHandle, FT_PURGE_TX | FT_PURGE_RX) == FT_OK;
        }

        // Synchronizes communication speed
        // RETURN: true synchronization succeeded
        public bool synchronize() {
            return true;
        }

        // Signal if the PC is ready (should map to DTR)
        // on: set to true to signal ready
        // RETURN: true on success
        public bool signal(bool on) {
            if (ftHandle == 0) return false;
            if (on) FT_SetDtr(ftHandle); else FT_ClrDtr(ftHandle);
            return true;
        }

    }
}
