using System;
using System.Collections.Generic;
using System.Text;

namespace IntelligentTransducer {

    // Basically the content of the communication packet
    class Transducer {
        public byte[] identity;
        public UInt16 model;
        public DateTime calibration;
        public DateTime expiry;
        public List<Channel> channels;

        public void fillFromPacket(TecnoComm.packetNET_UNIT packet) {
            identity = (byte[])packet.identity.Clone();
            model    = packet.model;
            calibration = packet.calibration;
            expiry = packet.expiry;
            channels = new List<Channel>();
            for(int i=0;i<packet.numberOfChannels;i++) channels.Add(null);
        }
    }

    // Basically the content of the communication packet
    class Channel {
        public UInt16 type;
        public UInt16 supply;
        public string unit;
        public byte measure;
        public byte[] exponents;
        string[] exponentNames = new string[] {
            "rad",
            "srad",
            "m",
            "kg",
            "s",
            "A",
            "K",
            "M",
            "C"
        };

        public void fillFromPacket(TecnoComm.packetNET_CHANNEL packet) {
            type = packet.transducer_type;
            supply = packet.supply;
            unit = packet.unit;
            measure = packet.measure;
            exponents = packet.measureExponents;
        }

        public string measureString() {
            StringBuilder sb = new StringBuilder();
            switch (measure) {
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_SI: break;
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_UU: sb.Append("Ratio"); break;
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_LOG10U: sb.Append("Log10"); break;
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_LOG10UU: sb.Append("Log10 ratio"); break;
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_DIGITAL: return "Digital";
                case TecnoComm.packetNET_CHANNEL.MEASURE_TYPE_ARBITRARY: return "Arbitrary";
            }
            for (int i = 0; i < exponents.Length; i++) {
                if (exponents[i] != 128) {
                    if (sb.Length!=0) sb.Append(" ");
                    sb.Append(exponentNames[i]);
                    if ((exponents[i] & 0x01) == 0) {
                        if (exponents[i]!=130) sb.Append((exponents[i] - 128) / 2);
                    }
                    else { sb.Append(exponents[i] - 128); sb.Append("/2"); }
                }          
            }
            return sb.ToString();
        }

    }
}
