using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace IT_READER {
    public partial class frmMain : Form {
        
        TecnoComm.communication communication = new TecnoComm.communication();

        public frmMain() {
            InitializeComponent();
        }

        private void btnSelectPort_Click(object sender, EventArgs e) {
            frmPortSelect form = new frmPortSelect();
            communication.freeReaderPort();
            form.ShowDialog();
            if (form.device == null) Close();
            else communication.setReaderPort(form.device);            
        }

        private void frmMain_Shown(object sender, EventArgs e) {
            btnSelectPort_Click(null, null);
        }

        IntelligentTransducer.Transducer transducer;

        private string byteArrayToString(byte[] a) {
            StringBuilder sb = new StringBuilder();
            foreach (byte b in a) {
                if (sb.Length!=0) sb.Append(" ");
                sb.Append(String.Format("{0:X2}",b));
            }
            return sb.ToString();
        }

        private void showTransducerData(IntelligentTransducer.Transducer transducer) {
            txtIdentity.Text = byteArrayToString(transducer.identity); 
            txtModel.Text=""+transducer.model;
            txtChannels.Text=""+transducer.channels.Count;
            txtCalibration.Text = String.Format("{0:dd-MMM-yyyy}", transducer.calibration);
            txtExpiry.Text = String.Format("{0:dd-MMM-yyyy}", transducer.expiry);
            lstChannel.Items.Clear();
            for (int i = 0; i < transducer.channels.Count; i++) {
                String s = i.ToString() + ": ";
                if (transducer.channels[i] != null) s += transducer.channels[i].unit;
                lstChannel.Items.Add(s);
            }            
        }

        private void enableRead(bool how) {
            btnRead.Enabled = how;
            chkAutoRead.Enabled=how;
            if (!how) {
                chkAutoRead.Checked = false;
                tmrRead.Enabled = false;
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e) {
            enableRead(false);
            TecnoComm.packetNET_UNIT pktUnitOut=new TecnoComm.packetNET_UNIT();
            TecnoComm.packetNET_UNIT pktUnitIn=new TecnoComm.packetNET_UNIT();

            TecnoComm.packetNET_CHANNEL pktChannel=new TecnoComm.packetNET_CHANNEL();

            try {
                communication.exchangePacket(pktUnitOut, pktUnitIn, 500);
                transducer = new IntelligentTransducer.Transducer();
                transducer.fillFromPacket(pktUnitIn);                
            }
            catch (TecnoComm.error error) {
                tslInfo.Text = error.Message;
                return;
            }

            TecnoComm.packetNET_CHANNEL pktChannelOut = new TecnoComm.packetNET_CHANNEL();
            TecnoComm.packetNET_CHANNEL pktChannelIn = new TecnoComm.packetNET_CHANNEL();

            try {
                for (int i = 0; i < transducer.channels.Count; i++) {
                    pktChannelOut.channel = (UInt16) i;
                    communication.exchangePacket(pktChannelOut, pktChannelIn, 500);
                    IntelligentTransducer.Channel channel = new IntelligentTransducer.Channel();
                    channel.fillFromPacket(pktChannelIn);
                    transducer.channels[i] = channel;
                }
                tslInfo.Text = "OK";
            }
            catch (TecnoComm.error error) {
                tslInfo.Text = error.Message;
                return;
            }
            showTransducerData(transducer);


        }

        private void lstChannel_SelectedIndexChanged(object sender, EventArgs e) {
            if (lstChannel.SelectedIndex >= 0) {
                IntelligentTransducer.Channel channel=transducer.channels[lstChannel.SelectedIndex];
                txtType.Text=channel.type.ToString();
                txtSupply.Text=channel.supply.ToString()+" mA";
                txtUnit.Text=channel.unit;
                txtMeasure.Text = channel.measureString();
                enableRead(true);
            }
            else {
                txtType.Text = "";
                txtSupply.Text = "";
                txtUnit.Text = "";
                txtMeasure.Text = "";
                enableRead(false);
            }
        }

        frmStats stats = null;

        private void btnStats_Click(object sender, EventArgs e) {
            if (stats == null || stats.IsDisposed) stats = new frmStats();
            stats.Visible = true;
        }

        private void btnRead_Click(object sender, EventArgs e) {
            TecnoComm.packetNET_READ packetReadOut=new TecnoComm.packetNET_READ();
            TecnoComm.packetNET_READ packetReadIn=new TecnoComm.packetNET_READ();

            try {
                packetReadOut.command=TecnoComm.packetNET_READ.CHANNEL_COMMAND_START;
                packetReadOut.channel = (UInt16)lstChannel.SelectedIndex;
                communication.exchangePacket(packetReadOut, packetReadIn, 5000);
                        
                int start=System.Environment.TickCount;
                packetReadOut.command = TecnoComm.packetNET_READ.CHANNEL_COMMAND_NONE;
                while ((packetReadIn.error & 0xFF00) == TecnoComm.packetNET_READ.CHANNEL_WAIT) {
                    packetReadOut.command = TecnoComm.packetNET_READ.CHANNEL_COMMAND_NONE;
                    communication.exchangePacket(packetReadOut, packetReadIn, 5000);
                    System.Threading.Thread.Sleep(500);
                    if (System.Environment.TickCount - start > 5000) {
                        lstRead.Items.Add("Measure not completed");
                        return;
                    }
                }

                if (packetReadIn.error == 0) {
                    StringBuilder sb = new StringBuilder();
                    sb.Append(packetReadIn.value.ToString());
                    if (txtMeasure.Text.Equals("K")) {
                        sb.Append(" = ");
                        sb.Append(packetReadIn.value - 273.15);
                        sb.Append("C");
                    }
                    lstRead.Items.Add(sb.ToString());
                }
                else lstRead.Items.Add("Error: " + String.Format("{0:X4}", packetReadIn.error));

                if (stats!=null) stats.addSample(packetReadIn.value);

                lstRead.SelectedIndex = lstRead.Items.Count - 1;
            }
            catch (TecnoComm.error error) {
                lstRead.Items.Add(error.Message);
                tslInfo.Text = error.Message;
                return;
            }



        }

        private void btnClear_Click(object sender, EventArgs e) {
            lstRead.Items.Clear();
        }
        
        private void chkAutoRead_CheckedChanged(object sender, EventArgs e)
        {
            if (chkAutoRead.Checked)
            {
                uint time;
                if (!UInt32.TryParse(edtAutoRead.Text,out time))
                {
                    time = 10;
                    edtAutoRead.Text = "10";
                }
                tmrRead.Interval = (int)time * 1000;
                tmrRead.Enabled = true;
            }
            else tmrRead.Enabled = false;
        }

        bool inTmrRead = false;

        private void tmrRead_Tick(object sender, EventArgs e)
        {
            if (inTmrRead) return;
            btnRead_Click(null, null);  
        }
        /*
        bool activating = false;

        private void frmMain_Activated(object sender, EventArgs e) {
            if (activating) return;
            if ((stats != null) && !stats.IsDisposed && !stats.TopMost) stats.TopMost = true;
            activating = true;
            this.Activate();
            this.Focus();
            activating = false;
        }

        private void frmMain_Deactivate(object sender, EventArgs e) {
            if (stats != null) stats.TopMost = false;
        }
        */


    }
}