﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace IT_READER {
    public partial class frmStats : Form {
        uint samples;
        uint errors;
        double maximum;
        double minimum;
        double sigma;
        double sigma2;

        double[] history;
        uint hisFirst;
        uint hisCount;        

        public frmStats() {
            InitializeComponent();
            history = new double[pnlHistory.Width];
            btnReset_Click(null, null);
        }

        private void updateDisplay() {
            // Errors
            txtErrors.ForeColor = errors == 0 ? Color.Blue : Color.Red;
            txtErrors.Text = "" + errors;
            // Statistics
            txtCount.Text = "" + samples;
            if (samples!=0) {
                txtMaximum.Text = ""+maximum;
                txtMinimum.Text = "" + minimum;
                double avg = sigma / samples;
                txtAverage.Text = "" + avg;
                double sd = Math.Sqrt(sigma2 / samples - avg * avg);
                txtSD.Text = "" + sd;
            }
            else {
                txtMaximum.Text = "";
                txtMinimum.Text = "";
                txtAverage.Text = "";
                txtSD.Text = "";
            }
            // Graphics
            pnlHistory.Invalidate();
        }

        public void addSample(double sample) {
            txtLast.Text = "" + sample;
            if (Double.IsNaN(sample) || Double.IsInfinity(sample)) {
                errors++;
            }
            else {
                if (samples == 0 || sample < minimum) {
                    minimum = sample;
                    txtMinimum.ForeColor = Color.Red;
                }
                else txtMinimum.ForeColor = Color.Blue;

                if (samples == 0 || sample > maximum) {
                    maximum = sample;
                    txtMaximum.ForeColor = Color.Red;
                }
                else txtMaximum.ForeColor = Color.Blue;

                sigma += sample;

                sigma2 += sample * sample;

                samples++;
            }

            if (hisCount < history.Length) {
                history[hisCount++] = sample;
            }
            else {
                history[hisFirst++] = sample;
                if (hisFirst >= history.Length) hisFirst = 0;
            }

            updateDisplay();
        }

        private void btnReset_Click(object sender, EventArgs e) {
            samples = errors = 0;
            maximum = minimum = sigma = sigma2 = 0.0;
            hisFirst = hisCount = 0;
            updateDisplay();
        }

        private void btnExit_Click(object sender, EventArgs e) {
            Visible = false;
        }

        private void pnlHistory_Paint(object sender, PaintEventArgs e) {
            double min = minimum;
            double max = maximum;
            if (min == max) { min = minimum - 0.5; max = maximum + 0.5; }
            Graphics g = e.Graphics;
            uint next = (hisCount + hisFirst) % (uint)history.Length;
            g.DrawLine(Pens.DarkGray, next, 0, next, pnlHistory.Height);
            for (uint i = 0; i < hisCount; i++) {
                uint im=(i+hisFirst) % (uint)history.Length;
                double d = history[im];
                if (Double.IsNaN(d) || Double.IsInfinity(d)) {
                    g.DrawLine(Pens.Red, im, 0, im, pnlHistory.Height);
                }
                else {
                    int y = (int)((d - min) / (max - min) * (pnlHistory.Height-10));
                    g.DrawRectangle(Pens.Blue, im, pnlHistory.Height-5-y, 2, 2);
                }
            }
        }

    }
}
